var vmc = {
	self: new VisualizerMicro(),

	init: function() {
		vmc.self.load(audio.src,vmc.loaded);
	},

	loaded: function(t) {
		console.log(t);
	},

	globalSettings: {
		color: 'rgba(0,255,0,1)',
		bgColor: 'rgba(0,0,0,1)',
		smooth: 0
	},
	globalSettingsRange: {
		smooth: [0,20]
	},

	draw: {
		waveform: {
			name: 'Waveform',
			id: 'waveform',
			out: function(){
				//clear canvas
				canvasCtx.fillStyle = vmc.globalSettings.bgColor;
				//canvasCtx.clearRect(0,0,canvas.width,canvas.height);
				canvasCtx.fillRect(0,0,canvas.width,canvas.height);
				

				//draw the lines
				canvasCtx.strokeStyle = vmc.globalSettings.color;
				canvasCtx.lineWidth = 1;
				canvasCtx.lineCap = "butt";
				canvasCtx.beginPath();
				canvasCtx.moveTo(0,(canvas.height * (vmc.draw.waveform.settings.vPos / 100)));
				//get the waveform
				var wf = vmc.self.getWaveform();

				//see how much we have to ignore
				var wfsp = Math.floor(wf.length / canvas.width);


				if(vmc.globalSettings.smooth > 0) {
					wf = valueSmoother(wf,wfsp + vmc.globalSettings.smooth);
				}
				//loop thru it all
				
				for(var i = 0; i < canvas.width; i++) {
					if(vmc.draw.waveform.settings.fill) {
						canvasCtx.moveTo(i,(canvas.height * (vmc.draw.waveform.settings.vPos / 100)));
					} 
					canvasCtx.lineTo(i,(canvas.height * (vmc.draw.waveform.settings.vPos / 100)) + (wf[i * wfsp] * vmc.draw.waveform.settings.sens));
				}
				canvasCtx.stroke();

			},
			settings: {
				vPos: 50,
				sens: 60,
				fill: false
			},
			settingsName: {
				vPos: 'Vertical Position',
				sens: 'Sensitivity',
				fill: 'Fill'
			},
			settingsRange: {
				vPos: [0,100],
				sens: [0,300]
			},
			settingsStep: {
				sens: 10
			}
				
		}, //close waveform

		circle: { //KEEP THIS NAME AS "circle" SO THAT SETTINGS FROM PREVIOUS VERSIONS ARE KEPT!!
			name: 'Circle Spectrum',
			id: 'spcircle',
			rotationOffsets: [],
			oldSettingLineCount: null,
			calcValues: function(sp){
				var skip = sp.length / vmc.draw.circle.settings.lineCount,
				fullCircle = Math.PI * 2,
				rotamt = (1 / Math.floor(sp.length / skip)) * fullCircle;
				vmc.draw.circle.rotationOffsets = [];

				for(var i = 0; i < Math.floor(sp.length / skip); i++) {
					vmc.draw.circle.rotationOffsets.push([
						Math.cos((i * rotamt)/*  - (fullCircle / 4) */),
						Math.sin((i * rotamt)/*  - (fullCircle / 4) */)
						/* Number(Math.cos(i * rotamt).toFixed(3)),
						Number(Math.sin(i * rotamt).toFixed(3)) */

					]);
				}
			},
			out: function() {
				//clear canvas
				canvasCtx.fillStyle = vmc.globalSettings.bgColor;
				//canvasCtx.clearRect(0,0,canvas.width,canvas.height);
				canvasCtx.fillRect(0,0,canvas.width,canvas.height);
				

				//draw the lines
				canvasCtx.strokeStyle = vmc.globalSettings.color;
				//canvasCtx.fillStyle = vmc.globalSettings.color;
				canvasCtx.lineWidth = vmc.draw.circle.settings.bWidth;

				if(vmc.draw.circle.settings.roundEnd) {
					canvasCtx.lineCap = "round";
				} else {
					canvasCtx.lineCap = "square"; //lmao
				}
				//canvasCtx.shadowColor = vmc.globalSettings.color;
				//canvasCtx.shadowBlur = 15;
				canvasCtx.beginPath();
				//get the data
				var sp = vmc.self.getSpectrum();

				//cut out if needed
				sp.splice(0,vmc.draw.circle.settings.btmFignore);
				sp.splice(-1 * vmc.draw.circle.settings.topFignore, vmc.draw.circle.settings.topFignore);
				if(vmc.globalSettings.smooth > 0) {sp = valueSmoother(sp,vmc.globalSettings.smooth);} //smooth
				if(vmc.draw.circle.settings.minVol > 0) {sp = spectrumMinRem(sp,vmc.draw.circle.settings.minVol);} //min vol

				if(vmc.draw.circle.settings.lineCount !== vmc.draw.circle.oldSettingLineCount) {
					vmc.draw.circle.calcValues(sp);
					vmc.draw.circle.oldSettingLineCount = vmc.draw.circle.settings.lineCount;
				}

						var skip = sp.length / vmc.draw.circle.settings.lineCount,
						scale = 0,
						scaleAmount = vmc.draw.circle.settings.space;

						if(vmc.draw.circle.settings.scale) {
							if(vmc.draw.circle.settings.scaleMinVol > 0) {
								var sctmp = (getave(sp) - (vmc.draw.circle.settings.scaleMinVol / 100)), tkaw = (vmc.draw.circle.settings.scaleMinVol / 100);
								if(sctmp < 0) {
									sctmp = 0;
								} else {
									sctmp = sctmp / (1 - tkaw);
								}
								scale = (scaleAmount * sctmp);
							} else {
								scale = scaleAmount * (getave(sp));
							}
						}

						canvasCtx.beginPath();
						for(var i = 0; i < Math.floor(sp.length / skip); i++) {
							var tbh = (vmc.draw.circle.settings.sens * sp[Math.floor(i * skip)]);
							
							var ofx = vmc.draw.circle.rotationOffsets[i][0], ofy = vmc.draw.circle.rotationOffsets[i][1];
							var ofxs = ofx * (vmc.draw.circle.settings.space + scale), ofxe = ofx * (vmc.draw.circle.settings.space + tbh + scale),
							ofys = ofy * (vmc.draw.circle.settings.space + scale), ofye = ofy * (vmc.draw.circle.settings.space + tbh + scale) ;
							canvasCtx.moveTo((canvas.width / 2) + ofxs, (canvas.height / 2) + ofys);
							canvasCtx.lineTo((canvas.width / 2) + ofxe, (canvas.height / 2) + ofye);
							
						}
						canvasCtx.stroke();
			},
			settings: {
				sens: 70,
				space: 30,
				bWidth: 3,
				lineCount: 40,
				roundEnd: true,
				scale: true,
				scaleMinVol: 0,
				topFignore: 852,
				btmFignore: 100,
				//topFignore: 0,
				//btmFignore: 0
				minVol: 35
			},
			settingsName: {
				sens: 'Sensitivity',
				space: 'Radius',
				style: 'Style',
				bWidth: 'Bar Width',
				//bSpace: 'Bar Spacing (Style 1)',
				lineCount: '# Of Bars',
				roundEnd: 'Rounded Line Caps',
				scale: 'Scale To Volume',
				scaleMinVol: 'Scale Min. Vol. (%)',
				topFignore: 'High Freq. Ignore',
				btmFignore: 'Low Freq. Ignore',
				minVol: 'Minimum Volume (%)'
			},
			settingsRange: {
				sens: [0,200],
				space: [1,100],
				bWidth: [1,30],
				lineCount: [3,64],
				//bSpace: [0,Infinity],
				scaleMinVol: [0,100],
				topFignore: [0,1024],
				btmFignore: [0,1024],
				minVol: [0,100],
			},
			settingsStep: {
				sens: 10,
				topFignore: 4,
				btmFignore: 4
			}
		}, //close circle

		spectrum: {
			name: 'Spectrum',
			id: 'spectrum',
			out: function(){
				//clear canvas
				canvasCtx.fillStyle = vmc.globalSettings.bgColor;
				//canvasCtx.clearRect(0,0,canvas.width,canvas.height);
				canvasCtx.fillRect(0,0,canvas.width,canvas.height);
				
		
				//draw the bars
				//get the spectrum
				var sp = vmc.self.getSpectrum();
				//cut out explicitly ignored frequencies
				sp.splice(0,vmc.draw.spectrum.settings.btmFignore);
				sp.splice(-1 * vmc.draw.spectrum.settings.topFignore, vmc.draw.spectrum.settings.topFignore);

				//smooth if user says so
				if(vmc.globalSettings.smooth > 0) {
					sp = valueSmoother(sp,vmc.globalSettings.smooth);
				}

				//also remove stuff below minimum, if user says so
				if(vmc.draw.spectrum.settings.minVol > 0) {sp = spectrumMinRem(sp,vmc.draw.spectrum.settings.minVol);}

				//debug only: test weighted ave for spectrum
				//sp = specweightedave(sp);

				canvasCtx.fillStyle = vmc.globalSettings.color;
				canvasCtx.strokeStyle = vmc.globalSettings.color;
				//see how much we have to ignore
				//var spsp = Math.floor(sp.length / (canvas.width / (vmc.draw.spectrum.settings.bWidth + vmc.draw.spectrum.settings.bSpace)));
				var spsp = sp.length / (canvas.width / (vmc.draw.spectrum.settings.bWidth + vmc.draw.spectrum.settings.bSpace));
				//loop thru it all
				
				canvasCtx.beginPath();
				canvasCtx.moveTo(-100,0); //out of bounds so it don't show up on the edge of the screen

				for(var i = 0; (i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth) < canvas.width; i++) {
					//var tbh = (vmc.draw.spectrum.settings.sens * sp[i * spsp]), height;
					var tbh = (vmc.draw.spectrum.settings.sens * sp[Math.floor(i * spsp)]), height;

					//determine height
					switch(vmc.draw.spectrum.settings.style) {
						case 0: 
						case 2:
							height = tbh; 
							break;
						case 1: 
							height = vmc.draw.spectrum.settings.cHeight; 
							tbh -= height;
							break;
						case 3:
							//clamp value, basically a kind of math.floor thing
							height = tbh - (tbh % (vmc.draw.spectrum.settings.cHeight + vmc.draw.spectrum.settings.bSpace));
							break;
							//following code is for previous loop function, instead of the overlay some bars function
							//height is unset here. clamp the tbh value according to the settings.
							tbh = tbh - (tbh % (vmc.draw.spectrum.settings.cHeight + vmc.draw.spectrum.settings.bSpace));
							break;
						default: height = 1; break; //just in case
					}
					
					//determine how 2 draw
					switch(vmc.draw.spectrum.settings.style) {
						case 0:
							if(tbh <= 0) {break;} //bro just dont draw if it's 0
							if(vmc.draw.spectrum.settings.round) {
								rectroundtop(
									(i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth),
									canvas.height - tbh,
									vmc.draw.spectrum.settings.bWidth,
									height
								);
								break;
							}
						case 1:
							if(tbh <= 0) {break;}
							canvasCtx.fillRect(
								(i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth),
								canvas.height - tbh,
								vmc.draw.spectrum.settings.bWidth,
								height
							);
							break;
						case 2:
							canvasCtx.lineTo((i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth),canvas.height - tbh);
							break;
						case 3:
							canvasCtx.fillRect(
								(i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth),
								canvas.height - height,
								vmc.draw.spectrum.settings.bWidth,
								height
							);

							break;

							//previous loop way thing
							/* for(var s3i = 0; s3i < tbh; s3i += (vmc.draw.spectrum.settings.cHeight + vmc.draw.spectrum.settings.bSpace)) { */
								/* if(vmc.draw.spectrum.settings.round) {
									canvasCtx.beginPath();
									canvasCtx.ellipse(
										(vmc.draw.spectrum.settings.bWidth / 2) + (i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth), //add some space at the left, origin draws at the center!
										(vmc.draw.spectrum.settings.cHeight / 2) + canvas.height - tbh + s3i,
										vmc.draw.spectrum.settings.bWidth / 2,
										vmc.draw.spectrum.settings.cHeight / 2,
										0,0, 2 * Math.PI
									);
									canvasCtx.fill();
									canvasCtx.beginPath();
								} else { */
									/* canvasCtx.fillRect(
										(i * vmc.draw.spectrum.settings.bSpace) + (i * vmc.draw.spectrum.settings.bWidth),
										canvas.height - tbh + s3i,
										vmc.draw.spectrum.settings.bWidth,
										vmc.draw.spectrum.settings.cHeight
										); */
								//}
							//}
							//break;
					}
						
				}


				//comment this entire "if" block if you want to enable the previous way of drawing this
				if(vmc.draw.spectrum.settings.style === 3) {
					//canvasCtx.fillStyle = '#f00';
					if(vmc.globalSettings.bgColor.indexOf('rgba') === -1) {
						canvasCtx.fillStyle = vmc.globalSettings.bgColor;
					} else {
						canvasCtx.fillStyle = rgbacolor2solid(vmc.globalSettings.bgColor);
					}
					var thht = (vmc.draw.spectrum.settings.bSpace + vmc.draw.spectrum.settings.cHeight);
					for(var i = 0; i < vmc.draw.spectrum.settings.sens + thht /* +1 more tick just in case */; i += thht) {
						canvasCtx.fillRect(0, canvas.height - i, canvas.width, vmc.draw.spectrum.settings.bSpace);
					}
				}

				canvasCtx.stroke();
		
			},
			settings: {
				sens: 200,
				//specIgnore: [0,0],
				topFignore: 852,
				btmFignore: 0,
				bWidth: 10,
				bSpace: 2,
				round: false,
				cHeight: 6,
				minVol: 0,
				style: 0
			},
			settingsName: {
				sens: 'Sensitivity',
				topFignore: 'High Freq. Ignore',
				btmFignore: 'Low Freq. Ignore',
				bWidth: 'Bar Width',
				bSpace: 'Bar Spacing',
				round: 'Round Top (Style 0)',
				cHeight: 'Cell Height (Style 1 + 3)',
				minVol: 'Minimum Volume (%)',
				style: 'Style'
			},
			settingsRange: {
				sens: [0,400],
				topFignore: [0,1024],
				btmFignore: [0,1024],
				bWidth: [1,40],
				bSpace: [0,40],
				cHeight: [1,40],
				minVol: [0,100],
				style: [0,3]
			},
			settingsStep: {
				sens: 10,
				topFignore: 4,
				btmFignore: 4
			}
		}, //close spectrum

		extravis1: {
			name: 'Extra Visualization 1',
			id: 'extravis1',
			out: function() {
				//clear canvas
				canvasCtx.fillStyle = vmc.globalSettings.bgColor;
				//canvasCtx.clearRect(0,0,canvas.width,canvas.height);
				canvasCtx.fillRect(0,0,canvas.width,canvas.height);
				

				//draw side waveforms
				canvasCtx.strokeStyle = vmc.globalSettings.color;
				canvasCtx.lineWidth = 1;
				canvasCtx.lineCap = "butt";
				//canvasCtx.shadowColor = vmc.globalSettings.color;
				//canvasCtx.shadowBlur = 15;
				//get the waveform
				var wf = vmc.self.getWaveform();
				if(vmc.globalSettings.smooth > 0) {
					wf = valueSmoother(wf,vmc.globalSettings.smooth);
				}
				var wfsp = Math.floor(wf.length / (canvas.width * 0.35));

				
				//loop thru it all
				
				//left one
				canvasCtx.beginPath();
				canvasCtx.moveTo(0,canvas.height * 0.5);
				var lineEnd = (canvas.width * 0.35),
				wflpoffset = 0;
				//canvasCtx.globalAlpha = 1,
				//onright = false;

				for(var i = 0; i < wf.length * 2; i += wfsp) {
					var percent = (i / wf.length),
					pos = percent * lineEnd;
					if(percent >= 1) {
						percent -= 1; //back to 0
						pos = canvas.width - (percent * lineEnd);
						if(wflpoffset === 0) {
							wflpoffset = wf.length;
							canvasCtx.moveTo(canvas.width,canvas.height * 0.5);
						}
					} 
					//canvasCtx.globalAlpha = 1 - percent;
					canvasCtx.lineTo(pos,(canvas.height * 0.5) + (wf[i - wflpoffset] * (vmc.draw.extravis1.settings.wfSens * (1 - percent))));
				}
				//canvasCtx.closePath();
				canvasCtx.stroke();


				//right one
/* 				canvasCtx.beginPath();
				canvasCtx.moveTo(canvas.width,canvas.height * 0.5);
				canvasCtx.globalAlpha = 1;
				for(var i = 0; i < wf.length; i += wfsp) {
					var percent = (i / wf.length);
					//canvasCtx.globalAlpha = 1 - percent;
					canvasCtx.lineTo(canvas.width - (percent * lineEnd),(canvas.height * 0.5) + (wf[i] * (vmc.draw.extravis1.settings.wfSens * (1 - percent))));
					canvasCtx.stroke();
				}
 */				//canvasCtx.globalAlpha = 1; //reset it

				//now the spectrum

				var sp = vmc.self.getSpectrum();
				//cut out explicitly ignored frequencies
				sp.splice(0,vmc.draw.extravis1.settings.btmFignore);
				sp.splice(-1 * vmc.draw.extravis1.settings.topFignore, vmc.draw.extravis1.settings.topFignore);

				//smooth and/or remove stuff if user says so
				if(vmc.globalSettings.smooth > 0) {sp = valueSmoother(sp,vmc.globalSettings.smooth);}
				if(vmc.draw.extravis1.settings.spMinVol > 0) {sp = spectrumMinRem(sp,vmc.draw.extravis1.settings.spMinVol);}

				//see what we leave out
				var twd = canvas.width * 0.3;//0.35 * 2 is 0.7, so we get 0.3 left
				var spsp = sp.length / twd; 
				
				//do the drawing
				canvasCtx.fillStyle = vmc.globalSettings.color;
				for(var i = 0; i < twd; i++) {
					var tbh = (vmc.draw.extravis1.settings.spSens * sp[Math.floor(i * spsp)]), extht = 0;
					if(vmc.draw.extravis1.settings.spReflect) {
						//draw reflection, if enabled
						extht = tbh / 2;
					}
					canvasCtx.fillRect(
						(canvas.width * 0.35) + (twd * (i / twd)),
						(canvas.height * (vmc.draw.extravis1.settings.spVpos / 100)) - tbh,
						1,
						tbh + extht
					);
				}
				if(vmc.draw.extravis1.settings.spReflect) {
				//if(false === true) { //this line for debug only!
						/* for(var i = 0; i < twd; i++) {
						var tbh = (vmc.draw.extravis1.settings.spSens * sp[Math.floor(i + spsp)]);
						
						canvasCtx.fillRect(
							(canvas.width * 0.35) + (twd * (i / twd)),
							(canvas.height * (vmc.draw.extravis1.settings.spVpos / 100)) + 1,
							1,
							tbh / 2
						);
					} */

					//overlay to darken it. gradient does not work :(
					//var grad = canvasCtx.createLinearGradient(0,0,0,vmc.draw.extravis1.settings.spSens / 2);
					//grad.addColorStop(0,'rgba(255,255,255,255)');
					//grad.addColorStop(1,vmc.globalSettings.bgColor);
					//canvasCtx.fillStyle = grad;
					canvasCtx.globalAlpha = 0.75;
					canvasCtx.fillStyle = vmc.globalSettings.bgColor;
					canvasCtx.fillRect(
						(canvas.width * 0.35),
						(canvas.height * (vmc.draw.extravis1.settings.spVpos / 100)) + 1,
						twd,
						vmc.draw.extravis1.settings.spSens / 2
					);
					canvasCtx.globalAlpha = 1;
					//vmc.draw.extravis1.settings.spSens
				}

							
					//canvasCtx.lineTo(i,(canvas.height * vmc.draw.spectrum.settings.vPos) + (wf[i * wfsp] * vmc.draw.spectrum.settings.sens));
			},
			settings: {
				wfSens: 40,
				spSens: 80,
				spMinVol: 40,
				spReflect: true,
				spVpos: 50,
				topFignore: 852,
				btmFignore: 100
			},
			settingsName: {
				wfSens: 'Waveform Sensitivity',
				spSens: 'Spectrum Sensitivity',
				spMinVol: 'Spectrum Min. Volume (%)',
				spReflect: 'Spectrum Reflection',
				spVpos: 'Spectrum Vertical Pos.',
				topFignore: 'Spec. High Freq. Ignore',
				btmFignore: 'Spec. Low Freq. Ignore'
			},
			settingsRange: {
				wfSens: [0,200],
				spSens: [0,160],
				spMinVol: [0,100],
				spVpos: [0,100],
				topFignore: [0,1024],
				btmFignore: [0,1024]
			},
			settingsStep: {
				wfSens: 10,
				spSens: 10,
				topFignore: 4,
				btmFignore: 4
			}
		}, //close extravis1

		wfcircle: {
			name: 'Circle Waveform',
			id: 'wfcircle',
			out: function() {
				//clear canvas
				canvasCtx.fillStyle = vmc.globalSettings.bgColor;
				//canvasCtx.clearRect(0,0,canvas.width,canvas.height);
				canvasCtx.fillRect(0,0,canvas.width,canvas.height);
				
		
				canvasCtx.strokeStyle = vmc.globalSettings.color;
				canvasCtx.lineWidth = 1;
				canvasCtx.lineCap = "butt";
				//canvasCtx.shadowColor = vmc.globalSettings.color;
				//canvasCtx.shadowBlur = 15;
				//get the waveform
				var wf = vmc.self.getWaveform();
				if(vmc.globalSettings.smooth > 0) {
					wf = valueSmoother(wf,vmc.globalSettings.smooth);
				}
				
				//main circle
				canvasCtx.beginPath();
				canvasCtx.moveTo(canvas.width * 0.5,canvas.height * 0.5);
				var fullCircle = Math.PI * 2, wfMirror = vmc.draw.wfcircle.settings.wfMirror;
				if(wfMirror) {wfMirror = 2;} else {wfMirror = 1;}
				var drawCircleLimit = wf.length * wfMirror;
		
				for(var i = 0; i < drawCircleLimit; i++) {
					var percent = (i / drawCircleLimit), wfSel = i;
		
					if(i >= wf.length) {wfSel = wf.length - (i - wf.length);}
		
					var curwf = wf[wfSel];
					if(vmc.draw.wfcircle.settings.wfAbs) {curwf = Math.abs(curwf);}
					var ofset = (curwf * vmc.draw.wfcircle.settings.wfSens) + vmc.draw.wfcircle.settings.centerRadius,
					curdeg = fullCircle * (percent - 0.25),
		
					x = (canvas.width / 2) + (Math.cos(curdeg) * ofset),
					y = (canvas.height / 2) + (Math.sin(curdeg) * ofset);
		
					if(i===0) {
						canvasCtx.moveTo(x,y);
					} else {
						canvasCtx.lineTo(x,y);
					}
				}
				canvasCtx.closePath();
				canvasCtx.stroke();
		
			},
			settings: {
				wfSens: 100,
				centerRadius: 1,
				wfMirror: false,
				wfAbs: false
			},
			settingsName: {
				wfSens: 'Waveform Sensitivity',
				centerRadius: 'Center Radius',
				wfMirror: 'Duplicate Waveform',
				wfAbs: 'Absolute Waveform'
			},
			settingsRange: {
				wfSens: [5,200],
				centerRadius: [0,100]
			},
			settingsStep: {
				wfSens: 5,
				centerRadius: 1
			}
		} //close wfcircle

		


	}
}

function valueSmoother(input,amount) { /* amount is number of points before and after. 1 means takes 1 point before and 1 point after. and so on. */
	var wftmp = [];

	for(var i = 0; i < input.length; i++) {
		//get the average of the points, then push to new array
		var smpt = 0, actlen = 0;
		for(var ii = amount * -1; ii < amount; ii++) {
			if(i + ii in input) {
				smpt += input[i + ii];
				actlen++;
			}
		}
		smpt = smpt / actlen;
		wftmp.push(smpt);
	}

	return wftmp;
}

function spectrumMinRem(sp,rm) { //rm should be integer
	//this handles the "minimum volume" thing
	var rm = rm / 100, nsp = sp;
	for(var i = 0; i < sp.length; i++) {
		nsp[i] = (sp[i] - rm);
		if(nsp[i] < 0) {
			nsp[i] = 0
		} else {
			nsp[i] = nsp[i] / (1 - rm);
		}
	}

	return nsp;
}

